'''
Modified deviation-based aiming strategy for flat receiver.
'''

import numpy as np
from sys import path
#from numpy import *

def aiming_flat(folder,h_rec,l_rec,C_aiming,csv,tower_h,Exp):
	title=np.array(['x', 'y', 'z', 'foc', 'aim x', 'aim y', 'aim z', 'm', 'm', 'm', 'm', 'm', 'm', 'm'])
	hst_info=np.loadtxt(csv,delimiter=',', skiprows=2) 
	num_hst=hst_info.size/7
	
	pos_and_aiming_new=np.array([])
	foc=hst_info[:,3]
	hst_info_ranked = hst_info[np.argsort(foc)[::1]]
	for j in range(len(hst_info)):
		lmax=np.max(hst_info_ranked[:,3])
		lmin=np.min(hst_info_ranked[:,3])
		li=hst_info_ranked[j,3]
		
		# for z coordinates
		d0=0.5*h_rec*C_aiming[0]*((lmax-li)/(lmax-lmin))**Exp[0]
		r0=round(random.uniform(0.,1),3)
		if r0<0.5:
			random_index0=-1
		elif r0 >=0.5:
			random_index0=1
		# the random index is -1 or +1
		hst_info_ranked[j,6]=tower_h+d0*random_index0 # z coordinates of aiming points
		
		# for x coordinates
		d1=0.5*h_rec*C_aiming[1]*((lmax-li)/(lmax-lmin))**Exp[1]
		r1=round(random.uniform(0.,1),3)
		if r1<0.5:
			random_index1=-1
		elif r1 >=0.5:
			random_index1=1
		hst_info_ranked[j,4]=d1*random_index1 # x coordinates of aiming points
		hst_info_ranked[j,3]=np.sqrt((hst_info_ranked[j,0]-hst_info_ranked[j,4])**2+(hst_info_ranked[j,1]-hst_info_ranked[j,5])**2+(hst_info_ranked[j,2]-hst_info_ranked[j,6])**2) # focal lenght
	pos_and_aiming_new=np.append(pos_and_aiming_new, hst_info_ranked)
	
	pos_and_aiming_new=np.append(title,pos_and_aiming_new)
	pos_and_aiming_new=pos_and_aiming_new.reshape(len(pos_and_aiming_new)/7, 7)
	csv_new=csv='%s/pos_and_aiming_new.csv' % folder # the output field file
	np.savetxt(csv_new, pos_and_aiming_new, fmt='%s', delimiter=',')
	
if __name__=='__main__':
	h_rec=2.75 # receiver height
	l_rec=2.75
	folder=path[0]
	csv='%s/sandia_hstat_coordinates_new.csv' % folder # the input field file
	tower_h=60. # that is the optical tower height

	C_aiming=np.array([0.5,0.5]) # in two directions, z and x, i==0 is z; i==1 is x
	Exp=np.array([1.5,1.5])
	aiming_flat(folder,h_rec,l_rec,C_aiming,csv,tower_h,Exp)



import numpy as np

class SunPosition:

    def __init__(self):
        pass


    def days(self, dd, mm):
        '''
        Arguement:
        dd - int, the day in the month
        mm - str, the month

        reference: J Duffie pp14, Table 1.6.1
        '''

        if mm=='Jan':
            days=dd

        elif mm=='Feb':
            days=31+dd

        elif mm=='Mar':
            days=59+dd

        if mm=='Apr':
            days=90+dd

        elif mm=='May':
            days=120+dd

        elif mm=='Jun':
            days=151+dd

        if mm=='Jul':
            days=181+dd

        elif mm=='Aug':
            days=212+dd

        elif mm=='Sep':
            days=243+dd

        if mm=='Oct':
            days=273+dd

        elif mm=='Nov':
            days=304+dd

        elif mm=='Dec':
            days=334+dd

        return days


    def declination(self, days, form='detail'):
        '''
        Reference: Solar Engineering of Thermal Processes, 4th edition, John A. Duffie and William A. Beckman, page 13

        declination angle: delta=23.45*sin(360*(284+day)/365)
 
        Arguement:
        day - int, day of the year (1-365)
        form - str, 'detail' or simple' model

        Return:
        delta - declination angle (deg)
        '''
        if form=='detail':
            B=float(days-1)*360./365.*np.pi/180.

            delta=(180./np.pi)*(0.006918 - 0.399912*np.cos(B) +0.070257*np.sin(B)- 0.006758*np.cos(2.*B) + 0.000907*np.sin(2.*B)- 0.002697*np.cos(3.*B) + 0.00148*np.sin(3.*B))

        else:
            delta=23.45*np.sin(360.*float(284+days)/365.*np.pi/180.) # deg

        return delta


    def solarhour(self, delta, latitude):

        '''        
        Reference: Solar Engineering of Thermal Processes, 4th edition, John A. Duffie and William A. Beckman, page 17

        Arguement:
        delta: declination angle- float, deg
        latitude: latitude angle: float, deg

        return: 
        hour: length of the daylight hour
        sunrise: the solar hour angle of the sunrise, deg

        '''

        sunset=np.arccos(-np.tan(latitude*np.pi/180.)*np.tan(delta*np.pi/180.))*180./np.pi # deg

        sunrise=-sunset

        hour=(sunset-sunrise)/15.
       
        return hour, sunrise


    def zenith(self, latitude, delta, omega):
        '''
        ref. eq.1.6.5
        Arguement:
        latitude: latitude angle, float, deg
        delta:  declination angle, float, deg
        omega: solar hour angle, float, deg

        return:
        theta: the zenith angle, float, deg

        '''             
        latitude*=np.pi/180.
        delta*=np.pi/180.
        omega*=np.pi/180.

        theta=np.arccos(np.cos(latitude)*np.cos(delta)*np.cos(omega)+np.sin(latitude)*np.sin(delta))*180./np.pi

        return theta
        
    def azimuth(self, latitude, theta, delta, omega):

        '''
        ref: eq. 1.6.6
        from South to West
        Arguement:
        latitude: latitude angle, deg
        delta: declination angle ,deg
        theta: zenith angle, deg
        omega: solar hour angle, deg

        return:
        phi: azimuth angle, deg, from South to West
        '''
        latitude*=np.pi/180.
        delta*=np.pi/180.
        theta*=np.pi/180.
    
        a1=np.cos(theta)*np.sin(latitude)-np.sin(delta)
        a2=np.sin(theta)*np.cos(latitude)
        b=a1/a2

        if abs(b+1.)<1e-10:
            phi=np.pi

        elif abs(b-1.)<1e-10:
            phi=0.
        else:
            phi=abs(np.arccos((np.cos(theta)*np.sin(latitude)-np.sin(delta))/(np.sin(theta)*np.cos(latitude)))) # unit radian

        if omega<0:
            phi=-phi

        phi*=180./np.pi

        return phi

    def convert_AZEL_to_declination_hour(self, theta, phi, latitude):

        '''
        Arguement:
        theta: zenith angle, deg
        phi: azimuth angle deg
        latitude: latitude latitude , deg

        return:
        delta: declination angle, deg
        omega: solar hour angle, deg
        '''      
        phi*=np.pi/180.
        theta*=np.pi/180.
        latitude*=np.pi/180.

        delta=np.arcsin(np.cos(theta)*np.sin(latitude)-np.cos(abs(phi))*np.sin(theta)*np.cos(latitude))

        omega=np.arccos((np.cos(theta)-np.sin(latitude)*np.sin(delta))/(np.cos(latitude)*np.cos(delta)))
        if phi<0:
            omega=-omega

        delta*=180./np.pi
        omega*=180./np.pi

        return delta, omega

    def convert_convention(self, tool, azimuth, zenith):
        '''
        tool: str, 'tracer', 'solstice', 'solartherm' or else TBD
        '''
        if tool=='solstice':
            # azimuth: from East to North 
            sol_azi=-(90.+azimuth)
            sol_ele=90.-zenith 
        elif tool=='solartherm':
            # azimuth: from North to Ease (clockwise)
            # ref Ali email (20/3/18)
            sol_azi=180.+azimuth
            sol_ele=90.-zenith  

        if isinstance(sol_azi, np.ndarray):
            for i in xrange(len(sol_azi)):
                azi=sol_azi[i]
                ele=sol_ele[i]
                if (azi>=360. or azi<0.):
                    sol_azi[i]=(azi+360.)%360.
                if ele<=1e-20:
                    sol_ele[i]=0.  
        else:
            if (sol_azi>=360. or sol_azi<0.):
                sol_azi=(sol_azi+360.)%360.
            if sol_ele<=1e-20:
                sol_ele=0.  
        
        return sol_azi, sol_ele        
                           

    def annual_angles(self, latitude, casefolder='NOTSAVE', nd=5, nh=5):
        '''
        Arguements:
        latitude: latitude of the location, deg
        nd: number of points in the declination movement ( -- suggest nd>=5)
        nh: number of points in the solar hours (half of the day)
        '''

        # declination angle (deg)  
        # -23.45 ~ 23.45
        DELTA=np.linspace(-23.45, 23.45, nd)
        #print 'declination', delta

        # the maximum solar hour angle
        # summer solstice
        if latitude>=0:
            # North hemisphere
            delta_summer=23.45
        else:
            # South hemishpere
            delta_summer=-23.45
        
        daymax, sunmax=self.solarhour(delta_summer, latitude)
        solartime=np.linspace(sunmax, -sunmax, nh*2-1)
        #print 'solar', solar

        # solar time
        time=12.+solartime/15
        #print time

        table=np.zeros(((nh*2-1+3)*(nd+4)))
        table=table.astype(str)
        for i in xrange(len(table)):
            table[i]=' '

        table=table.reshape(nh*2-1+3,nd+4)
        table[0,4]='Declination (deg)'
        table[1,0]='Lookup table'
        table[1,4:]=np.arange(1,nd+1)
        table[2,2]='Solar time (h)'
        table[2,3]='Hour angle (deg)'
        table[2,4:]=DELTA
        table[3:,1]=np.arange(1,nh*2-1+1)
        table[3:,2]=time
        table[3:,3]=solartime

        c=1 
        AZI=np.array([])
        ZENITH=np.array([])

        case_list=np.array(['Case','declination (deg)','solar hour angle (deg)', 'azimuth (deg) S-to-W ', 'zenith (deg)'])
        for i in xrange(len(DELTA)):
            delta=DELTA[i]
            hour, sunrise=self.solarhour(delta, latitude)
            sunset=-sunrise
            for j in xrange(len(solartime)):
                omega=solartime[j]
                if (omega>sunset or omega<sunrise):
                    table[3+j,4+i]='-' 
                else:
                    if omega<0:
 
                        table[3+j, 4+i]=' case %s'%(c)
                        table[-(1+j), 4+i]='***%s'%(c)

                        #zenith angle (morning)
                        theta=self.zenith(latitude, delta, omega)
                        # azimuth        
                        phi=self.azimuth(latitude, theta, delta, omega)
                        case_list=np.append(case_list, (c, delta, omega, phi, theta)) 

                        AZI=np.append(AZI, phi)
                        ZENITH=np.append(ZENITH, theta)

                        #zenith angle (afternoon)
                        theta=self.zenith(latitude, delta, -omega)
                        # azimuth        
                        phi=self.azimuth(latitude, theta, delta, -omega)
                        case_list=np.append(case_list, (c, delta, -omega, phi, theta)) 

                        c+=1
                    elif omega==0:
 
                        table[3+j, 4+i]=' case %s'%(c)

                        #zenith angle
                        theta=self.zenith(latitude, delta, omega)
                        # azimuth        
                        phi=self.azimuth(latitude, theta, delta, omega)
                        AZI=np.append(AZI, phi)
                        ZENITH=np.append(ZENITH, theta)

                        case_list=np.append(case_list, (c, delta, omega, phi, theta)) 
                        c+=1
                        
                                             
        case_list=case_list.reshape(len(case_list)/5,5)
        #azimuth=case_list[1:,-2].astype(float)
        #zenith=case_list[1:,-1].astype(float)

        if casefolder!='NOTSAVE':    
            np.savetxt(casefolder+'/table_view.csv', table, fmt='%s', delimiter=',')  
            np.savetxt(casefolder+'/annual_simulation_list.csv', case_list, fmt='%s', delimiter=',')          

        return AZI, ZENITH,table,case_list

    def annual_angles_field(self, latitude, casefolder='NOTSAVE', nd=5, nh=5):
        '''
        Arguements:
        latitude: latitude of the location, deg
        nd: number of points in the declination movement ( -- suggest nd>=5)
        nh: number of points in the solar hours (half of the day)
        '''

        # declination angle (deg)  
        # -23.45 ~ 23.45
        DELTA=np.linspace(-23.45, 23.45, nd)
        #print 'declination', delta

        # the maximum solar hour angle
        # summer solstice
        if latitude>=0:
            # North hemisphere
            delta_summer=23.45
        else:
            # South hemishpere
            delta_summer=-23.45
        
        daymax, sunmax=self.solarhour(delta_summer, latitude)
        solartime=np.linspace(sunmax, -sunmax, nh*2-1)
        #print 'solar', solar

        # solar time
        time=12.+solartime/15
        #print time

        table=np.zeros(((nh*2-1+3)*(nd+4)))
        table=table.astype(str)
        for i in xrange(len(table)):
            table[i]=' '

        table=table.reshape(nh*2-1+3,nd+4)
        table[0,4]='Declination (deg)'
        table[1,0]='Lookup table'
        table[1,4:]=np.arange(1,nd+1)
        table[2,2]='Solar time (h)'
        table[2,3]='Hour angle (deg)'
        table[2,4:]=DELTA
        table[3:,1]=np.arange(1,nh*2-1+1)
        table[3:,2]=time
        table[3:,3]=solartime

        c=1 
        AZI=np.array([])
        ZENITH=np.array([])

        case_list=np.array(['Case','declination (deg)','solar hour angle (deg)', 'azimuth (deg) S-to-W ', 'zenith (deg)'])
        for i in xrange(len(DELTA)):
            delta=DELTA[i]
            hour, sunrise=self.solarhour(delta, latitude)
            sunset=-sunrise
            for j in xrange(len(solartime)):
                omega=solartime[j]
                if (omega>sunset or omega<sunrise):
                    table[3+j,4+i]='-' 
                else:
  
                    table[3+j, 4+i]=' case %s'%(c)

                    #zenith angle
                    theta=self.zenith(latitude, delta, omega)
                    # azimuth        
                    phi=self.azimuth(latitude, theta, delta, omega)
                    AZI=np.append(AZI, phi)
                    ZENITH=np.append(ZENITH, theta)

                    case_list=np.append(case_list, (c, delta, omega, phi, theta)) 
                    c+=1
                        
                        
                       
        case_list=case_list.reshape(len(case_list)/5,5)
        #azimuth=case_list[1:,-2].astype(float)
        #zenith=case_list[1:,-1].astype(float)

        if casefolder!='NOTSAVE':    
            np.savetxt(casefolder+'/table_view.csv', table, fmt='%s', delimiter=',')  
            np.savetxt(casefolder+'/annual_simulation_list.csv', case_list, fmt='%s', delimiter=',')          

        return AZI, ZENITH,table,case_list[1:]
        


if __name__=='__main__':
    # example: PS10, summer solstice, solar noon
    latitude=37.44

    sun=SunPosition()
    dd=sun.days(21, 'Jun')
    delta=sun.declination(dd)
    print('Declination angle', delta)


    daytime,sunrise=sun.solarhour(delta, latitude)
    print('Day timeS', daytime)
    print('sun rise', sunrise)
    print('')

    omega=0. # solar noon
    theta=sun.zenith(latitude, delta, omega)
    phi=sun.azimuth(latitude, theta, delta, omega)
    print('elevation', 90.-theta)
    print('azimuth', phi)

    azi, zen, table,caselist=sun.annual_angles(latitude, hemisphere='North', casefolder='.',nd=5, nh=7)


    
    



        
        
        



from .aiming_strategy import *
from .cal_field import *
from .cal_layout import *
from .cal_sun import *
from .data_spectral import *
from .find_solstice import *
from .gen_vtk import *
from .gen_yaml import *
from .process_raw import *




import numpy as np
import sys
import matplotlib.pyplot as plt
from .cal_field import *
from .cal_sun import *
from .gen_vtk import gen_vtk

def radial_stagger(num_hst, width, height, hst_z, towerheight, R1, dsep=0., field='polar-half', savedir='.', plot=False):
    '''
    Ref. (Collado and Guallar, 2012), Campo: Generation of regular heliostat field.

    Generate a rather large field

    Arguements:
    num_hst    : int, number of heliostats
    width      : float, mirror width (m)
    height     : float, mirror height (m)
    hst_z      : float, the vertical location of each heliostat (m)
    towerheight: float, tower height (m)
    R1         : float, distance from the first row to the bottom of the tower (0, 0, 0)
    dsep       : float, separation distance (m)
    field      : str, 'polar-half' or 'surround-half' or 'polar' or 'surround' field, 
                       the 'half' option is for simulation a symmetric field
    savedir    : str, directory of saving the pos_and_aiming.csv

    Return:
    pos_and_aiming: (n, 7) array, position, focal length and aiming point of each generated heliostat
    '''
 
    # heliostat diagonal distantce
    DH=np.sqrt(height**2+width**2) 

    # distance between contiguous helistat center on the X and Y plane
    DM=DH+dsep

    # minimum radial increment
    delta_Rmin=0.866*DM

    # number of heliostats in the first row
    Nhel1 =int(2.*np.pi*R1/DM)


    # the total number of zones (estimated)
    #Nzones=int(np.log(5.44*3*(num_hst/az_rim*np.pi)/Nhel1**2+1)/np.log(4))+1

    X=np.array([])
    Y=np.array([])

    num=0
    i=0
    sys.stderr.write('DM '+repr(DM)+'\n')
    sys.stderr.write('dRm'+repr(delta_Rmin)+'\n')

    if field[-4:]=='half':

        while num<num_hst/2:
            Nrows= int((2.**(i))*Nhel1/5.44)
            Nhel=(2**(i))*Nhel1
            R=Nhel/2./np.pi*DM
            delta_az=2.*np.pi/Nhel
            sys.stderr.write('\nzone %d\n' % (i,))
            sys.stderr.write("R %f\n"%(R,))
            sys.stderr.write("rows %d\n"%(nrows,))
            sys.stderr.write("hst %d\n"%(Nhel,))
            sys.stderr.write("daz %f\n"%(delta_az,))

            for row in xrange(Nrows):

                r=R+float(row)*delta_Rmin

                for nh in xrange(Nhel):

                    if num<num_hst/2:   
               
                        if row%2==0:
                            # the odd row
                            azimuth=delta_az/2.+float(nh)*delta_az

                        else:
                            # the even row
                            azimuth=float(nh)*delta_az  
                 
                        xx=r*np.sin(azimuth)
                        yy=r*np.cos(azimuth)
    
                        if field=='polar-half':
                            if (xx>0 and yy>0):  
                                X=np.append(X, xx)
                                Y=np.append(Y, yy)
                                num+=1
                        elif field=='surround-half':
                            if (yy>0):  
                                X=np.append(X, xx)
                                Y=np.append(Y, yy)
                                num+=1
            i+=1

    else:
        while num<num_hst:
            Nrows= int((2.**(i))*Nhel1/5.44)
            Nhel=(2**(i))*Nhel1
            R=Nhel/2./np.pi*DM
            delta_az=2.*np.pi/Nhel
            
            sys.stderr.write("\nzone %d"%(i,))
            sys.stderr.write("R %f\n"%(R,))
            sys.stderr.write("rows %d\n"%(nrows,))
            sys.stderr.write("hst %d\n"%(Nhel,))
            sys.stderr.write("daz %f\n"%(delta_az,))		

            for row in xrange(Nrows):

                r=R+float(row)*delta_Rmin

                for nh in xrange(Nhel):
              
                    if num<num_hst:

                        if row%2==0:
                            # the odd row
                            azimuth=delta_az/2.+float(nh)*delta_az

                        else:
                            # the even row
                            azimuth=float(nh)*delta_az  
                 
                        xx=r*np.sin(azimuth)
                        yy=r*np.cos(azimuth)
             
                        if field=='polar':
                            if (yy>0):  
                                X=np.append(X, xx)
                                Y=np.append(Y, yy)
                                num+=1
                        elif field=='surround':
                                X=np.append(X, xx)
                                Y=np.append(Y, yy)
                                num+=1
            i+=1

    sys.stderr.write("\ntotal hst %d\n"%(num,))

    hstpos=np.zeros(num*3).reshape(num, 3)
    hstpos[:, 0]=X
    hstpos[:, 1]=Y
    hstpos[:,2]=hst_z

    aim_x=np.zeros(num)
    aim_y=np.zeros(num)
    aim_z=np.ones(num)*towerheight

    foc=np.sqrt((X-aim_x)**2+(Y-aim_y)**2+(hstpos[:,2]-aim_z)**2)

    pos_and_aiming=np.append(X, (Y, hstpos[:,2], foc, aim_x, aim_y, aim_z))
    title=np.array(['x', 'y', 'z', 'foc', 'aim x', 'aim y', 'aim z', 'm', 'm', 'm', 'm', 'm', 'm', 'm'])
    pos_and_aiming=pos_and_aiming.reshape(7,len(pos_and_aiming)/7)
    pos_and_aiming=np.append(title, pos_and_aiming.T)
    pos_and_aiming=pos_and_aiming.reshape(len(pos_and_aiming)/7, 7)
    
    np.savetxt('%s/pos_and_aiming.csv'%savedir, pos_and_aiming, fmt='%s', delimiter=',')

    if plot:
        fts=24
        plt.figure(dpi=100.,figsize=(12,9))
        plt.plot(X, Y, '.')
        plt.xlim(-1000, 1000)
        plt.ylim(-1000, 1000)
        plt.xticks(fontsize=fts)
        plt.yticks(fontsize=fts)
        plt.savefig(open('%s.png'%field,'w'), bbox_inches='tight')
        plt.close()

    return pos_and_aiming


if __name__=='__main__':
    
    pos_and_aim=radial_stagger(num_hst=4000, width=12.3, height=9.75, hst_z=1., towerheight=100, R1=87.5, dsep=0., field='polar', savedir='.', plot=True)

        




def gen_vtk(savedir, points, indices, norms, colormap=True, DATA=None):
    '''
    points - x, y, z
    indices - vertices of the triangle mesh
    DATA - dic, key is 'cosine' or 'atm' or others
    num - number of cell data 
    '''
    num_points=len(points.T)
    num_tri=len(indices)
    f=open(savedir, 'w')
    f.write('# vtk DataFile Version 2.0\n')
    f.write('test\n')
    f.write('ASCII\n')
    f.write('DATASET POLYDATA\n')
    f.write('POINTS %s double\n'%num_points)
    for i in xrange(num_points):
        x=points[0,i]
        y=points[1,i]
        z=points[2,i]
        f.write('%.8f %.8f %.8f\n'%(x, y, z))
    f.write('POLYGONS %s %s\n'%(num_tri, num_tri*4))
    for i in xrange(num_tri):
        id1=indices[i,0]
        id2=indices[i,1]
        id3=indices[i,2]
        f.write('3 %.0f %.0f %.0f\n'%(id1, id2, id3))  
    f.write('CELL_DATA %s\n'%(num_tri))
    f.write('NORMALS cell_normals float\n')

    for i in xrange(num_tri):
        nx=norms[i,0]
        ny=norms[i,1]
        nz=norms[i,2]
        
        f.write('%.8f %.8f %.8f\n'%(nx, ny, nz))  
        

    if colormap:    

        totalnum=len(DATA)         
        f.write('FIELD PrimaryData %s\n'%(totalnum))
        interest=DATA.keys()
        for m in interest:
            f.write('%s 1 %s double\n'%(m, num_tri))
            for i in xrange(num_tri):
                f.write('%.8f\n'%(DATA[m][i]))            

       
        f.close()


if __name__=='__main__':
    gen_vtk()




import numpy as np

from .data_spectral import SolarSpectrum, MirrorRhoSpectrum
import sys

def gen_yaml(DNI, sunshape, sunsize, hst_pos, hst_foc, hst_aims,hst_w, hst_h
		, rho_refl, slope_error, receiver, rec_param, rec_abs
		, outfile_yaml, outfile_recv
		, hemisphere='North', tower_h=0.01, tower_r=0.01,  spectral=False
		, medium=0, one_heliostat=False
):
    """
    Arguments:
    (1) the sun
    DNI: float, the direct normal irrandice of solar radiation (W/m2)
    sunshape: str, 'buie' or ' pillbox
    sunsize: float, pillbox half-angle (deg), or Buie CSR value 


    (2) the field
    hst_pos: (n, 3) array, heliostat positions (x, y, z)
    hst_foc: (n, 1) array, heliostat focal length
    hst_aims: (n, 3) array, heliostat aiming point(ax, ay, az)
    hst_w: float, heliostat mirror width (in x direction)
    hst_h: float, heliostat mirror height (in y direction)
    hst_z: float, heliostat center height (in z direction)
    rho_refl: float, reflector reflectivity
    slope_error: float, reflector surface slope error rms, (rad)
    tower_h: float, tower height
    tower_r: float, tower radius (a cylindrical shape tower)

    (3) the receiver
    receiver: str, 'flat', 'cylinder', or 'stl'
    rec_abs: float, receiver absorptivity
    rec_param: numpy array, each element contain the geometrical parameter of the correpsonding receiver
	    (1) flat receiver: index 0 is width, 1 is height, 2 is slices, 3- 6 is x, y, z and tilt angle (deg)
	    (2) cylinder receiver: index 0 is radius, 1 is height, 2 is slices
	    (3) stl: the directory of the stl file

    (4) others
    spectral: bool, if investigate spectral dependent performance 
    medium: float, if the atmosphere is surrounded by non-participant medium, medium=0; otherwise it is the extinction coefficient in m-1

    Return:
    Generation of the YAML inputs for SOLSTICE simulation
    """

    sys.stderr.write("Generating YAML file...\n")

    iyaml='' # the input yaml file

    # 
    ### Section (1)
    # set the spectral data: 
    # solar radiative intensity, refractive indexes, extinction coefficients, reflectivities
    #------------------------------
    if spectral:
        I_sun=SolarSpectrum()
        # CREATE the spectrum for the sun
        iyaml+='- spectrum: &solar_spectrum  \n'
        for i in range(0,len(I_sun)-1):
            iyaml+='  - {wavelength: %s, data: %s }\n' % (I_sun[i][0],I_sun[i][1])  
        i = len(I_sun)-1
        iyaml+='  - {wavelength: %s, data: %s }\n' % (I_sun[i][0],I_sun[i][1]) 
        iyaml+='\n'

        # CREATE the spectrum for the reflectivity (mirror)
        mirror_rho= MirrorRhoSpectrum()
        mirror_ref=mirror_rho
        for i in range(0,len(mirror_rho)):
            mirror_ref[i][0] = mirror_rho[len(mirror_rho)-1-i][0]/1000.
            mirror_ref[i][1] = mirror_rho[len(mirror_rho)-1-i][1]/100.
        mirror_ref.append([4,0.9])
        iyaml+='- spectrum: &%s  \n' % 'ref_mirror'
        for i in range(0,len(mirror_ref)-1):
            iyaml+='  - {wavelength: %15.8e, data: %15.8e }\n' % (float(mirror_ref[i][0]),float(mirror_ref[i][1])) 
        i = len(mirror_ref)-1
        iyaml+='  - {wavelength: %15.8e, data: %15.8e }\n' % (float(mirror_ref[i][0]),float(mirror_ref[i][1])) 
        iyaml+='\n'

    # 
    ### Section (2)
    # set the medium types: 
    # air, glass, vacuum, etc. gathering spectral data
    #------------------------------
    #
    if medium>1e-99:
        #TODO need data/info of extinction coefficient as a function of spectral
        ke_air = I_sun
        for i in range(0,len(I_sun)):
            ke_air[i][1] = medium
        iyaml+='- spectrum: &%s  \n' % 'airkext'
        for i in range(0,len(ke_air)-1):
            iyaml+='  - {wavelength: %s, data: %s }\n' % (ke_air[i][0],ke_air[i][1])
        i = len(ke_air)-1
        iyaml+='  - {wavelength: %s, data: %s }\n' % (ke_air[i][0], ke_air[i][1])
        iyaml+='\n'
        #
        # Creation of the sun and atmosphere
        #
    if spectral:
        iyaml+='- sun: {dni: %15.8e, spectrum: *solar_spectrum, %s: {half_angle: %6.4f}}\n' % (DNI, sunshape,sunsize) 
    else:
        iyaml+='- sun: {dni: %15.8e, %s: {half_angle: %6.4f}}\n' % (DNI, sunshape,sunsize)  
    if medium>1e-99:
        iyaml+='- atmosphere: {extinction: *airkext}\n' 
        iyaml+='\n'

	       
    # 
    ### Section (3)
    # set the materials
    # (gathering media)
    # occultant material, mirror specular material, receiver material, virtual target
    #------------------------------
    #
    # CREATE an occultant material
    r_f = 0. # front
    r_b = 0. # and back reflectivity
    iyaml+='- material: &%s\n' % 'material_black'
    iyaml+='   front:\n'
    iyaml+='     matte: {reflectivity: %6.4f }\n' % r_f    
    iyaml+='   back:\n'
    iyaml+='     matte: {reflectivity: %6.4f }\n' % r_b
    iyaml+='\n'
    #
    # CREATE a specular material
    r_f= rho_refl # front
    r_b = 0.      # and back reflectivity
    iyaml+='- material: &%s\n' % 'material_mirror'
    iyaml+='   front:\n'
    if spectral:
        iyaml+='     mirror: {reflectivity: *%s, slope_error: %15.8e }\n' % ('ref_mirror', slope_error ) 
    else:
        iyaml+='     mirror: {reflectivity: %6.4f, slope_error: %15.8e }\n' % (r_f, slope_error) 

    iyaml+='   back:\n'
    iyaml+='     matte: {reflectivity: %6.4f }\n' % r_b 
    iyaml+='\n'
    #
    # CREATE a material for the target
    r_f = 1.-rec_abs # front
    r_b = 1.-rec_abs # and back reflectivity
    iyaml+='- material: &%s\n' % 'material_target'
    iyaml+='   front:\n'
    iyaml+='     matte: {reflectivity: %6.4f }\n' % r_f    
    iyaml+='   back:\n'
    iyaml+='     matte: {reflectivity: %6.4f }\n' % r_b
    iyaml+='\n'
    #
    # CREATE a virtual material for the calculation of spillage
    iyaml+='- material: &%s\n' % 'material_virtual'
    iyaml+='   virtual:\n'
    iyaml+='\n'


    # 
    ### Section (4)
    # set the geometries
    # (gathering shapes and materials)
    # the tower, the receiver, the heliostat
    #------------------------------
    #
    # Tower Geometry
    # (cylindrical shape)
    #
    slices = 10 # slices for the envelop circle
    iyaml+='- geometry: &%s\n' % 'tower_g' 
    iyaml+='  - material: *%s\n' % 'material_black' 
    #iyaml+='    transform: { translation: %s, rotation: %s }\n' % ([0, 0, h_tow*0.5], [0, 90, 0]) 
    iyaml+='    cylinder: {height: %7.3f, radius: %7.3f, slices: %d }\n' % (tower_h, tower_r, slices) 
    iyaml+='\n'
    #
    # Receiver Geometry
    #
    if receiver=='flat':
        geom, rec_entt, rcv = flat_receiver(rec_param, hemisphere)
        iyaml+=geom

    elif receiver=='cylinder':
        geom, rec_entt, rcv = cylindrical_receiver(rec_param, hemisphere)
        iyaml+=geom

    elif receiver=='stl':
        rec_entt, rcv=STL_receiver(rec_param, hemisphere)
    #
    # Heliostats Geometry
    #
    if one_heliostat:
        hst_x=np.r_[hst_pos[0]]
        hst_y=np.r_[hst_pos[1]]
        hst_z=np.r_[hst_pos[2]]
        aim_x=np.r_[hst_aims[0]] 
        aim_y=np.r_[hst_aims[1]]
        aim_z=np.r_[hst_aims[2]]
        num_hst=1
        hst_foc=np.r_[hst_foc]
    else:
        hst_x=hst_pos[:,0]
        hst_y=hst_pos[:,1]
        hst_z=hst_pos[:,2]
        aim_x=hst_aims[:,0]
        aim_y=hst_aims[:,1]
        aim_z=hst_aims[:,2]
        num_hst=len(hst_x)
    slices = 4 # slices for the envelop circle
    pts_hst = [ [-hst_w*0.5, -hst_h*0.5], [-hst_w*0.5, hst_h*0.5], [hst_w*0.5, hst_h*0.5], [hst_w*0.5,-hst_h*0.5] ]
    # CREATE a reflective facet (mirror)
    for i in range(0,num_hst):
        name_hst_g = 'hst_g_'+str(i)
        iyaml+='- geometry: &%s\n' % name_hst_g 
        iyaml+='  - material: *%s\n' % 'material_mirror' 
        #iyaml+='    transform: { translation: %s, rotation: %s }\n' % ([hst_x[i], hst_y[i], hst_z[i]], [0, 0, 0]) )
        iyaml+='    parabol: \n'
        iyaml+='      focal: %s\n' % hst_foc[i]
        iyaml+='      clip: \n'  
        iyaml+='      - operation: AND \n'
        iyaml+='        vertices: %s\n' % pts_hst
        iyaml+='      slices: %d\n' % slices  

    # CREATE the pylon "pylon_g" geometry cylindrical shape
    h_pyl = 0.001 # pylon height
    r_pyl = 0.2 # pylon radius
    slices = 4 # slices for the envelop circle
    iyaml+='- geometry: &%s\n' % 'pylon_g' 
    iyaml+='  - material: *%s\n' % 'material_black' 
    iyaml+='    transform: { translation: %s, rotation: %s }\n' % ([0, 0, -h_pyl*3], [0, 90, 0]) 
    iyaml+='    cylinder: {height: %7.3f, radius: %7.3f, slices: %d }\n' % (h_pyl,r_pyl,slices) 
    #   

    # 
    ### Section (5)
    # set the templates
    # (programming objects gathering geometries or pivot and geometries)
    #------------------------------
    # CREATE the heliostat templates
    for i in range(0,num_hst):    
        name_hst_t = 'hst_t_'+str(i)
        iyaml+='- template: &%s\n' % name_hst_t 
        name_hst_n = 'hst_'+ str(i)
        iyaml+='    name: %s\n' % name_hst_n 
        iyaml+='    primary: 0\n'   
        iyaml+='    geometry: *pylon_g\n'
        iyaml+='    children: \n' 
        iyaml+='    - name: pivot\n'
        iyaml+='      zx_pivot: {target: {position: %s}} \n' % ([aim_x[i],aim_y[i],aim_z[i]]) 
        iyaml+='      children: \n'
        iyaml+='      - name: reflect_surface\n'
        iyaml+='        primary: 1\n'
        iyaml+='        transform: {rotation: [-90,0,0]} \n'   
        name_hst_g = 'hst_g_'+str(i)
        iyaml+='        geometry: *%s\n' % name_hst_g 

    # 
    ### Section (6)
    # set the entities
    # (gather templates to be created and active in the scene)
    #------------------------------
    #
    # receiver entities
    iyaml+=rec_entt
    #
    # tower entities
    iyaml+='\n- entity:\n'
    iyaml+='    name: tower_e\n'
    iyaml+='    primary: 0\n' 
    iyaml+='    transform: { translation: %s, rotation: %s }\n' % ([0, -tower_r, tower_h*0.5], [0, 0, 0]) 
    iyaml+='    geometry: *%s\n' % 'tower_g'    
    #
    # heliostat entities from the template
    for i in range(0,num_hst):
        name_e ='H_'+str(i)
        name_hst_t = 'hst_t_'+str(i)
        iyaml+='\n- entity:\n'
        iyaml+='    name: %s\n' % name_e
        iyaml+='    transform: { translation: %s, rotation: %s }\n' % ([hst_x[i], hst_y[i], hst_z[i]], [0, 0, 0]) 
        iyaml+='    children: [ *%s ]\n' % name_hst_t    

    with open(outfile_yaml,'w') as f:
        f.write(iyaml)

    with open(outfile_recv,'w') as f:
        f.write(rcv) 


def flat_receiver(rec_param, hemisphere='North'):
    '''
    hemishpere : 'North' or 'South' hemisphere of the earth where the field located
                if North: the field is in the positive y direction
                if South: the field is in the negtive y direction
                this will influence:
                 (1) the setting of the receiver tilt angle, 
                     if the front surface always facing to the field is desirable
                 (2) the position of the virtual target
    '''
    rec_w=rec_param[0]
    rec_h=rec_param[1]
    slices=rec_param[2]
    x=rec_param[3]
    y=rec_param[4]
    z=rec_param[5]
    tilt=rec_param[6]
    # receiver tilt angle:
    # 0 is vertical
    # the standby posiion of a plane in solstice is normal points to the +z axis
    # rotation anagle, positive is anti-clockwise

    geom=''
    pts=[ [-rec_w*0.5, -rec_h*0.5], [-rec_w*0.5, rec_h*0.5], [rec_w*0.5, rec_h*0.5], [rec_w*0.5,-rec_h*0.5] ]

    geom+='- geometry: &%s\n' % 'target_g'
    geom+='  - material: *%s\n' % 'material_target'
    geom+='    plane: \n'
    geom+='      clip: \n' 
    geom+='      - operation: AND \n'
    geom+='        vertices: %s\n' % pts
    geom+='      slices: %d\n' % slices 
    geom+='\n'

    # CREATE a receiver entity from "target_g" geometry (primary = 0)
    entt=''
    entt+='\n- entity:\n'
    entt+='    name: target_e\n'
    entt+='    primary: 0\n'
    if hemisphere=='North':
        entt+='    transform: { translation: %s, rotation: %s }\n' % ([x, y, z], [-90.-tilt, 0, 0]) 
    else:
        entt+='    transform: { translation: %s, rotation: %s }\n' % ([x, y, z], [90.+tilt, 0, 0]) 
    entt+='    geometry: *%s\n' % 'target_g'

    # CREATE a virtual target entity from "target_g" geometry (primary = 0)
    pts = [ [-rec_w*10., -rec_h*10.], [-rec_w*10., rec_h*10.], [rec_w*10., rec_h*10.], [rec_w*10.,-rec_h*10.] ]
    slices = 4
    entt+='\n- entity:\n'
    entt+='    name: virtual_target_e\n'
    entt+='    primary: 0\n'
    if hemisphere=='North':
        entt+='    transform: { translation: %s, rotation: %s }\n' % ([x, y-5., z], [-90.-tilt, 0, 0])
    else:
        entt+='    transform: { translation: %s, rotation: %s }\n' % ([x, y+5., z], [90.+tilt, 0, 0])
    entt+='    geometry: \n' 
    entt+='      - material: *%s\n' % 'material_virtual' 
    entt+='        plane: \n'
    entt+='          clip: \n'    
    entt+='          - operation: AND \n'
    entt+='            vertices: %s\n' % pts
    entt+='          slices: %d\n' % slices  

    rcv=''
    rcv+='- name: target_e \n' 
    rcv+='  side: %s \n' % 'FRONT_AND_BACK'
    rcv+='  per_primitive: %s \n' % 'INCOMING_AND_ABSORBED'
    rcv+='- name: virtual_target_e \n'
    rcv+='  side: %s \n' % 'FRONT'
    rcv+='  per_primitive: %s \n' % 'INCOMING'
    
    return geom, entt, rcv
    


def cylindrical_receiver(rec_param, hemisphere='North'):
    '''
    hemishpere : 'North' or 'South' hemisphere of the earth where the field located
                if North: the field is in the positive y direction
                if South: the field is in the negtive y direction
                this will influence:
                 (1) the setting of the receiver tilt angle, 
                     if the front surface always facing to the field is desirable
                 (2) the position of the virtual target
    '''
    rec_r=rec_param[0]
    rec_h=rec_param[1]
    slices=rec_param[2]
    x=rec_param[3]
    y=rec_param[4]
    z=rec_param[5]

    geom=''
    geom+='- geometry: &%s\n' % 'target_g'
    geom+='  - material: *%s\n' % 'material_target'
    geom+='    cylinder: \n'
    geom+='      height: %s\n'%rec_h 
    geom+='      radius: %s\n'%rec_r 
    geom+='      slices: %d\n' % slices 
    geom+='\n'

    # CREATE a receiver entity from "target_g" geometry (primary = 0)
    entt=''
    entt+='\n- entity:\n'
    entt+='    name: target_e\n'
    entt+='    primary: 0\n'

    entt+='    transform: { translation: %s, rotation: %s }\n' % ([x, y, z], [0., 0., 0.]) 

    entt+='    geometry: *%s\n' % 'target_g'

    # CREATE a virtual target entity from "target_g" geometry (primary = 0)
    Vsize=100.
    pts = [ [-rec_h*Vsize, -rec_h*Vsize], [-rec_h*Vsize, rec_h*Vsize], [rec_h*Vsize, rec_h*Vsize], [rec_h*Vsize,-rec_h*Vsize] ]
    slices = 4
    entt+='\n- entity:\n'
    entt+='    name: virtual_target_e\n'
    entt+='    primary: 0\n'

    entt+='    transform: { translation: %s, rotation: %s }\n' % ([x, y, z+rec_h/2.+1], [-180., 0, 0])
 
    entt+='    geometry: \n' 
    entt+='      - material: *%s\n' % 'material_virtual' 
    entt+='        plane: \n'
    entt+='          clip: \n'    
    entt+='          - operation: AND \n'
    entt+='            vertices: %s\n' % pts
    entt+='          slices: %d\n' % slices  

    rcv=''
    rcv+='- name: target_e \n' 
    rcv+='  side: %s \n' % 'FRONT_AND_BACK'
    rcv+='  per_primitive: %s \n' % 'INCOMING_AND_ABSORBED'
    rcv+='- name: virtual_target_e \n'
    rcv+='  side: %s \n' % 'FRONT'
    rcv+='  per_primitive: %s \n' % 'INCOMING'
    
    return geom, entt, rcv

    

def STL_receiver(rec_param, hemisphere='North'):
    '''
    hemishpere : 'North' or 'South' hemisphere of the earth where the field located
                if North: the field is in the positive y direction
                if South: the field is in the negtive y direction
                this will influence:
                 (1) the setting of the receiver tilt angle, 
                     if the front surface always facing to the field is desirable
                 (2) the position of the virtual target
    '''

    rec_w=rec_param[0].astype(float) # for creating the virtual target
    rec_h=rec_param[1].astype(float)
    stlfile=rec_param[2] # directory of the stl file
    x=rec_param[3].astype(float)
    y=rec_param[4].astype(float)
    z=rec_param[5].astype(float)
    tilt=rec_param[6].astype(float) # need to figure out the initial mesh orientation

    # CREATE a receiver entity from a STL file 
    entt=''
    entt+='\n- entity:\n'
    entt+='    name: STL_receiver_e\n'
    entt+='    primary: 0\n'
    if hemisphere=='North':

        entt+='    transform: { translation: %s, rotation: %s }\n' % ([x, y, z], [-90.-tilt, 0, 0]) 
    else:
        # if it is the mesh model of the bladed receiver at CSIRO
        entt+='    transform: { translation: %s, rotation: %s }\n' % ([x, y, z], [180.+tilt, 0, 0]) 
    entt+='    geometry:\n'
    entt+='    - material: *material_target\n'
    entt+='      transform: {translation: [0, 0, 0], rotation: [0, 0, 0]}\n'
    entt+="      stl : {path: %s }  \n"%(stlfile)
 

    # CREATE a virtual target entity from "target_g" geometry (primary = 0)
    pts = [ [-rec_w*10., -rec_h*10.], [-rec_w*10., rec_h*10.], [rec_w*10., rec_h*10.], [rec_w*10.,-rec_h*10.] ]
    slices = 4
    entt+='\n- entity:\n'
    entt+='    name: virtual_target_e\n'
    entt+='    primary: 0\n'
    if hemisphere=='North':
        entt+='    transform: { translation: %s, rotation: %s }\n' % ([x, y-5., z], [-90.-tilt, 0, 0])
    else:
        entt+='    transform: { translation: %s, rotation: %s }\n' % ([x, y+5., z], [90.+tilt, 0, 0])
    entt+='    geometry: \n' 
    entt+='      - material: *%s\n' % 'material_virtual' 
    entt+='        plane: \n'
    entt+='          clip: \n'    
    entt+='          - operation: AND \n'
    entt+='            vertices: %s\n' % pts
    entt+='          slices: %d\n' % slices  

    rcv=''
    rcv+='- name: STL_receiver_e \n' 
    rcv+='  side: %s \n' % 'FRONT_AND_BACK'
    rcv+='  per_primitive: %s \n' % 'INCOMING_AND_ABSORBED'
    rcv+='- name: virtual_target_e \n'
    rcv+='  side: %s \n' % 'FRONT'
    rcv+='  per_primitive: %s \n' % 'INCOMING'

    return entt, rcv










    #------------------------------



import numpy as np
#from tracer.models.heliostat_field import solar_vector
#import matplotlib.pyplot as plt
from scipy.spatial import Delaunay
from .gen_vtk import gen_vtk

class FieldPF:

    def __init__(self, receiver_norm=np.r_[0,1,0]):
        '''
        Evaluation of field initial performance
        1. cosine factors: sun - heliostat
        2. another cosine factors: receiver -heliostat
        
        azimuth: float, the solar azimuth angle, from South to West
        zenith: float, the solar zenith angle, 0 from vertical
        receiver_normal: (3,) array, normal vector of the receiver aperature 

        '''
        #self.sun_vec=self.get_solar_vector(azimuth, zenith)
        self.rec_norm=receiver_norm.reshape(3,1)



    def get_solar_vector(self, azimuth, zenith):
        """
        Calculate the solar vector using elevation and azimuth.

        Arguments:
        azimuth - the sun's azimuth, in deg, 
	        from South increasing towards to the West
        zenith - angle created between the solar vector and the Z axis, in deg.

        Returns: a 3-component 1D array with the solar vector.
        """
        #copied from tracer.models.heliostat_field import solar_vector
        #TODO change it to Solstice convetion if necessary

        azimuth*=np.pi/180.
        zenith*=np.pi/180.

        sun_z = np.cos(zenith)
        sun_y=-np.sin(zenith)*np.cos(azimuth)
        sun_x=-np.sin(zenith)*np.sin(azimuth)
        sun_vec = np.r_[sun_x, sun_y,sun_z] 


        return sun_vec

    def get_normals(self, towerheight, hstpos, sun_vec):

        tower_vec=-hstpos
        tower_vec[:,-1]+=towerheight
        tower_vec/=np.sqrt(np.sum(tower_vec**2, axis=1)[:,None])

        hst_norms=sun_vec+tower_vec
        hst_norms/=np.sqrt(np.sum(hst_norms**2, axis=1)[:,None])        
        return hst_norms

    def get_rec_view(self, towerheight, hstpos):
        # angle between normal of the receiver apterture and the RH
        # RH is the vector from the receiver to the heliostat (i.e. -tower_vec)

        tower_vec=-hstpos
        tower_vec[:,-1]+=towerheight
        tower_vec/=np.sqrt(np.sum(tower_vec**2, axis=1)[:,None])

        view=np.arccos(np.dot(-tower_vec, self.rec_norm))
        view=view.flatten()
        vis_idx=(view<1.5) # the heliostat that can be seen by the receiver

        return vis_idx


    def get_cosine(self,hst_norms, sun_vec):
 
        cos_factor=np.sum(hst_norms*sun_vec, axis=1)

        return cos_factor

    def heliostat(self, width, height ):
        '''
        the local coordinate of the heliostat
        '''
        x=np.linspace(-width/2., width/2., 2)
        y=np.linspace(-height/2., height/2., 2)
        
        xx, yy=np.meshgrid(x, y)
        coords=np.column_stack([xx.ravel(),yy.ravel()])   
        tri=Delaunay(coords).simplices
        #plt.figure(1) 
        #plt.triplot(coords[:,0], coords[:,1], tri)   
        #plt.show()
        return coords, tri

    def view_heliostats(self, width, height, normals, hstpos):
        
        coord1, tri1=self.heliostat(width, height)

        ele=len(tri1)
        nc=len(coord1)

        num_hst=len(hstpos)

        COORD=np.zeros(num_hst*nc*3).reshape(num_hst*nc, 3)
        TRI=np.zeros(num_hst*ele*3).reshape(num_hst*ele, 3)

        norm_x=normals[:,0]
        norm_y=normals[:,1]
        norm_z=normals[:,2]          
  
        for i in xrange(num_hst):            
         
            TRI[i*ele: (i+1)*ele]=tri1+i*nc

            trans = np.eye(4)
            trans[:3,3] = hstpos[i]

            x=coord1[:,0]
            y=coord1[:,1]
            z=float(hstpos[i,2])*np.ones(nc)  

            cd=np.vstack((x, y, z, np.ones(nc)))   
            cd_t=np.dot(trans, cd)

            xx=cd_t[0]
            yy=cd_t[1]
            zz=cd_t[2]

            COORD[i*nc: (i+1)*nc,0]=xx
            COORD[i*nc: (i+1)*nc,1]=yy
            COORD[i*nc: (i+1)*nc,2]=zz

        #plt.figure(1) 
        #plt.triplot(COORD[:,0], COORD[:,1], TRI)   
        #plt.show()        
        return COORD, TRI, ele, nc

    def plot_cosine(self, savename,pm):
        x=self.hstpos[:,0]
        y=self.hstpos[:,1]
        z=self.hstpos[:,2]
        av=(self.view<np.pi/2.)
        plt.figure(1)
        cm = plt.cm.get_cmap('rainbow')
        cs=plt.scatter(x[av], y[av], c=self.cosine_factor[av], cmap=cm,s=30)
        plt.colorbar(cs)
        plt.title('Cosine factors\n Tower height %s m'%pm )
        plt.savefig(open(savename, 'w'),dpi=500, bbox_inches='tight')
        plt.close()	


    def plot_select(self, savename,pm):
        x=self.hstpos[:,0]
        y=self.hstpos[:,1]
        z=self.hstpos[:,2]

        plt.figure(1)
        fig,ax1=plt.subplots()
        cm = plt.cm.get_cmap('rainbow')
     
        av=(self.view<1.5)
        cs=plt.scatter(x[av], y[av], c=self.view[av], cmap=cm)
        plt.colorbar(cs)
        nv=(self.view>=1.5)
        plt.scatter(x[nv], y[nv], c='gray')
        plt.title('Receiver view \n tilt %s deg'%pm )
        #plt.title('Receiver view \n Tower height %s m'%pm )
        plt.savefig(open(savename, 'w'),dpi=500, bbox_inches='tight')
        plt.close()	
       
def rotx(ang):
    """Generate a homogenous trransform for ang radians around the x axis"""
    s = np.sin(ang); c = np.cos(ang)
    return np.array([
        [1., 0, 0, 0],
        [0, c,-s, 0],
        [0, s, c, 0],
        [0, 0, 0, 1.]
    ])

def roty(ang):
    """Generate a homogenous trransform for ang radians around the y axis"""
    s = np.sin(ang); c = np.cos(ang)
    return np.array([
        [c, 0, s, 0],
        [0, 1., 0, 0],
        [-s,0, c, 0],
        [0, 0, 0, 1.]
    ])

def rotz(ang):
    """Generate a homogenous trransform for ang radians around the z axis"""
    s = np.sin(ang); c = np.cos(ang)
    return np.array([
        [c,-s, 0, 0],
        [s, c, 0, 0],
        [0, 0, 1., 0],
        [0, 0, 0, 1.]
    ])

def translate(x=0, y=0, z=0):
    """Generate a homogenous transform for translation by x, y, z"""
    return np.array([
        [1., 0, 0, x],
        [0, 1., 0, y],
        [0 ,0, 1., z],
        [0, 0, 0, 1.]
    ])
          

if __name__=='__main__':
    #pos_and_aiming=np.loadtxt('/media/yewang/Work/svn_ye/Solstice-tutorial/cases/2-Validation/layout.csv', delimiter=',', skiprows=2)
    pos_and_aiming=np.loadtxt('./pos_and_aiming.csv', delimiter=',', skiprows=2)
    pos=pos_and_aiming[:,:3]
    aim=pos_and_aiming[:,4:]
    azimuth=np.r_[0.]
    zenith=np.r_[12.]
    field=FieldPF(azimuth, zenith,np.r_[0,1,0])
    sun_vec=field.get_solar_vector(azimuth, zenith)
    norms=field.get_normals(towerheight=70., hstpos=pos, sun_vec=sun_vec)
    #field.heliostat(10, 8)
    COORD, TRI, ele, nc=field.view_heliostats(10., 8., norms, pos)
    cos=field.get_cosine(towerheight=70., hstpos=pos)
    savedir='./field.vtk'
    COS=np.repeat(cos, ele)
    DATA={'cos':COS}
    NORMS=np.repeat(norms, ele, axis=0)
    gen_vtk(savedir, COORD.T, TRI, NORMS, True, DATA)
        
